#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include <sys/soundcard.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <sphinx2/fbs.h>
#include <sphinx2/ad.h>
#include <sphinx2/cont_ad.h>

int16 pcmu_to_l16(unsigned int ulawbyte);

#ifdef __cplusplus
}
#endif

#include "./asr.h"
int au;

/*
 * Caculate the SNR value
 * Excerpted from "radio.cpp" by Kundan
 */
#define SQR(x) (x)*(x)
double EnergyLevelPcmu (unsigned char *dataBuf, int dataLen)
{
  int i, n;
  double sum_x, avg_x;

  n= dataLen;

  /* Following portion of the code is copied from HW2 */
  for (sum_x=0, i=0; i < n; i++) {
    //sum_x += pcmu_l16(dataBuf[i]);
    sum_x += abs(pcmu_l16(dataBuf[i]));
  }
  avg_x = sum_x/(double)n;

  for (sum_x=0, i=0; i < n; i++) {
    sum_x += SQR(pcmu_l16(dataBuf[i]) - avg_x);
  }
  avg_x = sum_x/(double)n;

  return (avg_x);
}

double EnergyLevelL16 (int16 *dataBuf, int dataLen)
{
  int i, n;
  double sum_x, avg_x;

  n= dataLen;

  /* Following portion of the code is copied from HW2 */
  for (sum_x=0, i=0; i < n; i++) {
    sum_x += dataBuf[i];
  }
  avg_x = sum_x/n;

  for (sum_x=0, i=0; i < n; i++) {
    sum_x += SQR(dataBuf[i] - avg_x);
  }
  avg_x = sum_x/n;

  return (avg_x);
}

int main(int argc, char *argv[])
{

  int c;
  char arg_line[200];
  char *asrarg_file = NULL;
  FILE *asrarg_fp = NULL;
  FILE *log_fp;

  asrarg_file = argv[1];

  /** ASR argument file is required */
  if(asrarg_file == NULL){
    cerr << "Usage: " << argv[0] << " <ASR arg file>" << " " << endl;
    exit(1);
  }

  /** Read argument file for Sphinx ASR Engine */
  char *opt, *val, *asr_argv[100];
  int i, j, len, asr_argc;
  if(!(asrarg_fp = fopen(asrarg_file, "r"))){
    cerr << "Can't find ASR argument file" << asrarg_file << endl;
    exit(1);
  }

  asr_argc = 0;
  while(1){
    if (fgets(arg_line, sizeof(arg_line), asrarg_fp) == NULL)
      break;
    len = strlen(arg_line);
    if (len >= 1) {
      arg_line[len-1] = 0;
      if (arg_line[0] == '#')
        continue; /* skip comment */
     } else
       continue;

    len = strlen(arg_line);
    opt = arg_line;
    for(i=0, val=NULL; i<len; i++){
      if(arg_line[i] == ' '){
        arg_line[i] = 0;
        val = &arg_line[i+1];
        break;
      }
    }
    if( val == NULL)
      continue;

     asr_argv[asr_argc++] = strdup(opt);
     asr_argv[asr_argc++] = strdup(val);
  }
  if(asrarg_fp != NULL)
    fclose(asrarg_fp);

  for(i=0,j=1; i<asr_argc-1; i+=2,j++)
    fprintf(stdout, "%d: %s %s\n", j, asr_argv[i], asr_argv[i+1]);

  int format = AFMT_MU_LAW, stereo = 0, speed = 8000, duplex = 1;
  //int format = AFMT_S16_LE, stereo = 0, speed = 8000, duplex = 1;

  if((au = open("/dev/audio", O_RDONLY, NULL)) < 0){
  //if((au = open("/dev/dsp", O_RDONLY, NULL)) < 0){
    perror("open audio device");
    exit( -1);
  }
  if(ioctl(au, SNDCTL_DSP_SETFMT, &format) == -1){
    perror("SETFMT");
    exit( -1);
  }
  if(ioctl(au, SNDCTL_DSP_STEREO, &stereo) == -1){
    perror("STEREO");
    exit( -1);
  }
  if(ioctl(au, SNDCTL_DSP_SPEED, &speed) == -1){
    perror("SPEED");
    exit( -1);
  }
  if(ioctl(au, SNDCTL_DSP_SETDUPLEX, &duplex) == -1){
    perror("SETDUPLEX");
    exit( -1);
  }

  /** Initialize ASR Engine */
  if(fbs_init ( asr_argc, asr_argv)){
    fprintf(stderr, "main: failed: fbs_init\n");
    exit( -1);
  }
  
  u_char recv_buf[ RECV_BUF_SIZE];
  int16 l16_data[RECV_BUF_SIZE];

  /** Voice related */
  int rcnt, sil_cnt, sil_total, utt_total;
  bool talkspurt, sound;
  double el;
  int sil_dur_byte = SILENCE_DURATION_SEC * 8000; 
  int max_utt_byte = MAX_UTT_SEC * 8000; 
  int loaduse;    
  
  /** ASR related */
  char *hyp, tmp_msg[MAX_ASR_LENGTH];
  char *mesg;
  int32 asr_err, frm;

  /** Open log file */
  if(!(log_fp = fopen("audio.dat", "w"))){
    cerr << "Can't open log file audio.dat" << endl;
    exit(1);
  }

  /** Set logging directry */
  uttproc_set_rawlogdir("./log");

  //////////////////
  // ASR Main Loop
  while(1)
  {
    uttproc_begin_utt(NULL);
    talkspurt = false;
    utt_total = sil_cnt = sil_total = 0;

    fprintf(stdout, "Listening..\n");

    while(1){

      /* Read audio data */
      loaduse = read(au, recv_buf, 160);
      //loaduse = read(au, l16_data, 320);

      /* Write log data */
      fwrite(recv_buf, 160, 1, log_fp); 
      //fwrite(l16_data, 320, 1, log_fp); 

/***
      for(i=0; i<100; i++)
        fprintf(stdout, "%d ", recv_buf[i]);
      fprintf(stdout, "\n");
***/

      /** Silence Detection:
       * Assummig we're receiving pcmu from sipc,
       * The threshold is defined by VAT_THRESHOLD
       */
      rcnt = loaduse;

      //el = EnergyLevelPcmu( (unsigned char *)recv_buf, rcnt);
      //el = EnergyLevelL16( l16_data, rcnt);

      //if( el > 200.0){ /* PCM16 */
      //if( el > 3600.0){
      if(1){
        sound = true;
        //fprintf( stdout, "AsrRtpRecvThread: SOUND: pkt %d bytes pl=%d bytes el=%10.2f\n",
                         //loaduse, rcnt, el);
      } else {
        //fprintf( stdout, "AsrRtpRecvThread: SILENCE: pkt %d bytes pl=%d bytes el=%10.2f\n",
                         //loaduse, rcnt, el);
        sound = false;
      }
      
      /** POLICY:
       * A voice slot length is defined by MAX_UTT_SEC.
       * The silence in the begining of a slot is discarded,
       * but once sound is detected, silences between words are
       * accumulated into ASR engine. If the silence is
       * longer than SILENCE_DURATION_SEC, that is regarded as
       * the end of a talkspurt.
       */
      if(sound){
        talkspurt = true;

        /* reset silence timer */
        sil_cnt = 0;

      } else {
        if(talkspurt){
          /* accumulate silence after sound */
          sil_cnt += rcnt;

        } else {
          /* accumlate silence in non-talkspurt */
          sil_total += rcnt;

          /* no sound was detected within the slot */
          if(sil_total > max_utt_byte)
            break;
          else
            continue;
        }
      }

      /**  convert pcmu to liner 16 bits */
      //ad_mu2li(l16_data, recv_buf, rcnt);
      for(i=0; i<rcnt; i++)
        l16_data[i] = pcmu_to_l16((unsigned int)recv_buf[i]);

      /* Write log data */
/**
      fwrite(l16_data, 2*rcnt, 1, log_fp); 
**/
      /** Accumulate raw audio to ASR engine */
      uttproc_rawdata(l16_data, rcnt, 0);
      //uttproc_rawdata(l16_data, rcnt/2, 0);
      utt_total += rcnt;
      //fprintf(stdout, "AsrRtpRecvThread: ASR accumulated %d samples\n", rcnt);

      /* the end of the talkspurt */
      if(sil_cnt > sil_dur_byte) {
        fprintf(stdout, "END of the talkspurt: SILENCE %d bytes %d sec\n", sil_cnt, SILENCE_DURATION_SEC);
        break;
      }

      /**
       *  Anyway, the accumulated sound reached to
       *  the max. length, proceed to ASR processing.
       *  This happens if the speaker keeps talking without
       *  silence longer than SILENCE_DURATION_SEC.
       */
      if(utt_total > max_utt_byte) {
        fprintf(stdout, "REACHED Max. Length %d bytes\n", max_utt_byte);
        break;
      }

    } /* while utterance or less than Max. length */

    /** ASR Processing */
    if(talkspurt){
      uttproc_end_utt();
      asr_err = uttproc_result(&frm, &hyp, 1);
      if( asr_err < 0){
        fprintf(stderr, "Failed: AsrRtpRecvThread: uttproc_result\n");
        exit(-1);
      }
      fprintf(stdout, "Talkspurt: %d bytes, %3.1f sec, frm=%d, ASR=%s\n",
                      utt_total, utt_total/8000.0, frm, hyp);
      if(strlen(hyp)<=1)
        strcpy(tmp_msg, "Unrecognizable speech");
      else
        strcpy(tmp_msg, hyp);

    } else {
      uttproc_abort_utt();
      fprintf(stdout, "AsrRtpRecvThread: Silence: %d bytes %3.1f sec, NO-ASR\n",
                      sil_total, sil_total/8000.0);
      sprintf(tmp_msg, ".....silence %d sec", MAX_UTT_SEC);

    }

    /* Callback application to send as IM */
    mesg = strdup(tmp_msg);
    fprintf(stdout, "You said: [%s]\n", mesg);
    free(mesg);

  }

}




