# Columbia SIP makefile fragment for simvoice
# Copyright 1999-2000 by Columbia University; all rights reserved 

simvoice_dir := simvoice
sphinx_dir := /usr/local/lib

LIBECI = -libmeci50
LIBSPHINX = $(sphinx_dir)/libsphinx2ad.a $(sphinx_dir)/libsphinx2.a $(sphinx_dir)/libsphinx2fe.a

SIMVOICE_MODULES := docs

# Include the description for each sub-module
include $(patsubst %,$(topsrcdir)/$(simvoice_dir)/%/module.mk,$(SIMVOICE_MODULES))


SIMVOICE_BIN := \
  $(simvoice_dir)/simvoice

SIMVOICE_SRC := \
  $(simvoice_dir)/main.cpp \
  $(simvoice_dir)/pcmu_l16.c \
  $(simvoice_dir)/tts.cpp \
  $(simvoice_dir)/asr.cpp

SIMVOICE_HEADERS := \
  $(simvoice_dir)/rtp.h \
  $(simvoice_dir)/pcmu_l.h \
  $(simvoice_dir)/types.h \
  $(simvoice_dir)/simv.h

SIMVOICE_MISC := \
  $(simvoice_dir)/module.mk \
  $(simvoice_dir)/simvoice.dsp

SIMVOICE_MISCDIST := \
  $(simvoice_dir)/README \
  $(simvoice_dir)/simvoice.license


SIMVOICE_NEEDED_LIBRARIES_LIST := \
	$(simvoice_dir)/simvoice-needed-shared-libraries

SIMVOICE_NEEDED_LIBRARIES_SOURCES := \
	$(simvoice_dir)/simvoice-needed-shared-libraries-sources

SIMVOICE_OBJ := \
	$(patsubst %.cpp,%.o,$(filter %.cpp,$(SIMVOICE_SRC))) \
	$(patsubst %.c,%.o,$(filter %.c,$(SIMVOICE_SRC)))

SIMVOICE_DEP := \
        $(SIMVOICE_OBJ:.o=.d)

SIMVOICE_MISCCLEAN := \
	$(SIMVOICE_NEEDED_LIBRARIES_LIST)

# Include directories required locally.
$(simvoice_dir)-CFLAGS := $(SIPAPI_INC) $(DBAPI_INC) $(LIBSIP_INC) $(LIBCINE_INC) $(LIBRTP_INC)

SIMVOICE_LIBS := $(SIPAPI_LIB) $(LIBSIP_LIB) $(DBAPI_LIB) $(LIBCINE_LIB) \
	$(LIBRESPARSE_LIB) $(LIBDICT_LIB)  \
	$(LDAPLIBS) $(DBAPI_LIB) \
        $(MYSQLLIBS) $(ZLIBLIBS) $(RESOLVLIBS) \
	$(TLSLIBS) $(NISLIBS) $(SOCKLIBS) \
        $(LIBRTP_LIB) $(LIBECI) $(LIBSPHINX)

SIMVOICE_DEPENDENCIES := $(SIPAPI) $(LIBSIP) $(LIBCINE) $(LIBRESPARSE) \
                      $(LIBDICT) $(DBAPI) $(LIBRTP)
SIMVOICE_REQS :=


$(SIMVOICE_NEEDED_LIBRARIES_SOURCES): $(simvoice_dir)/simvoice
	@echo "Building list of non-system shared libraries..."
	rm -f $(SIMVOICE_NEEDED_LIBRARIES_SOURCES)
	$(LDD) $(simvoice_dir)/simvoice | \
		sed -n -e '/not found/ s/^/XXX SET LD_LIBRARY_PATH: /p;' \
			-e '/=>/ s|^.* \([^ ]*/[^ ]*\).*$$|\1|p' | \
			egrep -v '^(/usr)?/lib/' \
			> $(SIMVOICE_NEEDED_LIBRARIES_SOURCES); \
			true
	if grep XXX $(SIMVOICE_NEEDED_LIBRARIES_SOURCES) > /dev/null; \
	then \
		echo "Missing shared library; can't continue"; \
		cat $(SIMVOICE_NEEDED_LIBRARIES_SOURCES); \
		rm -f $(SIMVOICE_NEEDED_LIBRARIES_SOURCES); \
		exit 1; \
	fi

$(SIMVOICE_NEEDED_LIBRARIES_LIST): $(SIMVOICE_NEEDED_LIBRARIES_SOURCES)
	@echo "Building local names of non-system shared libraries..."
	rm -f $(SIMVOICE_NEEDED_LIBRARIES_LIST)
	@simvoice_needed_libraries=`cat $(SIMVOICE_NEEDED_LIBRARIES_SOURCES)`; \
	for library in $$simvoice_needed_libraries; \
	do\
		echo $(simvoice_dir)/`basename $$library`; \
	done > $(SIMVOICE_NEEDED_LIBRARIES_LIST)

# Do not add this --- all: $(SIMVOICE)
all: simvoice
.PHONY: simvoice
simvoice: $(SIMVOICE_REQS) $(SIMVOICE_DEP) $(SIMVOICE_BIN)

# SIMVOICE uses libsip++ == SIPAPI

$(simvoice_dir)/simvoice: $(SIMVOICE_OBJ) $(SIMVOICE_DEPENDENCIES)
	@echo "creating application $@..."
	$(CXX) -g -O2 -o $@ -w $(SIMVOICE_INCS) $(SIMVOICE_OBJ) \
	$(LDFLAGS) $(SIMVOICE_LIBS) $(LIBS)

BIN	  += $(SIMVOICE_BIN)
SRC	  += $(SIMVOICE_SRC)
MISC      += $(SIMVOICE_MISC)
MISCDIST  += $(SIMVOICE_MISCDIST)
MISCCLEAN += $(SIMVOICE_OTHER_MISCCLEAN)

SIMVOICE_DIST := $(SIMVOICE_SRC) $(SIMVOICE_HEADERS) $(SIMVOICE_MISC) $(SIMVOICE_MISCDIST)

SIMVOICE_BINDIST := $(SIMVOICE_MISCDIST) $(SIMVOICE_BIN)

SIMVOICE_MISC_DLLS := 

SIMVOICE_BIN_WIN32 := \
  $(simvoice_dir)/Debug/simvoice.exe \
  $(simvoice_dir)/Release/simvoice.exe

SIMVOICE_BINDIST_WIN32 := $(SIMVOICE_BIN_WIN32) $(SIMVOICE_MISCDIST) \
		      $(SIMVOICE_MISC_DLLS)

.PHONY: simvoice-dist simvoice-bindist simvoice-bindist-win32 copy-simvoice-shared-libraries

copy-simvoice-shared-libraries: $(SIMVOICE_NEEDED_LIBRARIES_SOURCES)
	@echo "Copying non-system shared libraries..."
	@simvoice_needed_libraries=`cat $(SIMVOICE_NEEDED_LIBRARIES_SOURCES)`; \
	if test "x$$simvoice_needed_libraries" != "x"; \
	then\
		echo cp $$simvoice_needed_libraries $(simvoice_dir); \
		cp $$simvoice_needed_libraries $(simvoice_dir); \
	fi


SIMVOICE_DIST_NAME := simvoice-$(VERSION)

SIMVOICE_DIST_FILES := \
	$(TOP_DIST) \
	$(LIBCINE_DIST) $(LIBSIP_DIST) $(LIBRESPARSE_DIST) \
	$(LIBDICT_DIST) \
	$(DBAPI_DIST) \
	$(WIN32_DIST) \
	$(SIPAPI_DIST) \
	$(SIMVOICE_DIST) \
	$(SIMVOICE_DOCS_DIST) \
	$(DOC_DIST) \
	$(EXTRA_DIST_FILES)

simvoice-dist: win32-dist
	$(MKDIST) $(topsrcdir) $(SIMVOICE_DIST_NAME) $(SIMVOICE_DIST_FILES)

SIMVOICE_BINDIST_NAME := simvoice-$(VERSION)-$(SYSNAME)

SIMVOICE_BINDIST_FILES := \
		$(TOP_BINDIST) \
		$(LIBCINE_BINDIST) \
		$(LIBDICT_BINDIST) \
		\
		$(SIMVOICE_BINDIST) \
		$(SIMVOICE_DOCS_BINDIST) \
		$(DOC_BINDIST) \
		`cat $(SIMVOICE_NEEDED_LIBRARIES_LIST)` \
		$(EXTRA_BINDIST_FILES)

simvoice-bindist:: $(SIMVOICE)  copy-simvoice-shared-libraries \
			$(SIMVOICE_NEEDED_LIBRARIES_LIST)
	$(MKDIST) $(topsrcdir) $(SIMVOICE_BINDIST_NAME) $(SIMVOICE_BINDIST_FILES)


SIMVOICE_BINDIST_WIN32 := $(SIMVOICE_MISCDIST) \
		$(simvoice_dir)/Debug/simvoice.exe \
                $(simvoice_dir)/Release/simvoice.exe


SIMVOICE_BINDIST_WIN32_NAME := simvoice-$(VERSION)-Win32

SIMVOICE_BINDIST_WIN32_FILES := \
		$(TOP_BINDIST) \
		$(LIBCINE_BINDIST) \
		$(LIBDICT_BINDIST) \
		\
		$(SIMVOICE_BINDIST_WIN32) \
		$(SIMVOICE_DOCS_BINDIST) \
		$(DOC_BINDIST) \
		$(EXTRA_BINDIST_FILES)

simvoice-bindist-win32::
	$(MKDIST_WIN32) $(topsrcdir) $(SIMVOICE_BINDIST_WIN32_NAME) \
	$(SIMVOICE_BINDIST_WIN32_FILES)

simvoice-help: 
	@echo "SIMVOICE (Test User Agent) Make Targets"
	@echo " make simvoice                Make simvoice executable on Unix."
	@echo " make simvoice-dist           Make simvoice source distribution."
	@echo " make simvoice-bindist        Make simvoice binary distribution for Unix."
	@echo " make simvoice-bindist-win32  Make simvoice binary distribution for Win32."

help: simvoice-help
