#ifdef __cplusplus
extern "C" {
#endif
  
#include <pthread.h>
#include <eci.h> /* IBM ViaVoice */

#include "rtp_api.h"
#include "rtp_highlevel.h"

#include "pcmu_l.h"
#include "error.h"
#include "dstring.h"

#ifdef __cplusplus
}
#endif

#include <string>
#include "iptelapi.h"

/** GLOBALS */

enum simv_system_status { 
 SIMV_NORMAL,
 SIMV_ACCEPT_PEND,
 SIMV_REJECT_PEND,
 SIMV_HANGUP_PEND
};

/* SIP IM */
#define MAX_IM_LENGTH 1024

/* ASR: Silence detection energy threshold */

//#define VAT_THRESHOLD 3500.0 /* HOME */
#define VAT_THRESHOLD 50000.0 /* CLIC */

/* silence duration to determine the end of talkspurt */
#define SILENCE_DURATION_SEC 1

/*
 * silence duration of voice
 * that should be notified to IM user
 */
#define SILENCE_NOTIFY_SEC 5

/* Max. talkspurt for Sphinx is 30sec, let's limit it to 15sec */
#define MAX_UTT_SEC 15

/* buffer length of max. result of ASR */
#define MAX_ASR_LENGTH 1024

/* RTP send size */ 
#define RTP_SEND_SIZE 160

/*
 * buffer length of partial result of TTS
 * must be multiple of RTP_SEND_SIZE above
 */
#define TTS_BUF_SIZE 32000

/* RTP: Payload type */
#define PT_MULAW_8 0

/* RTP: Max. size of UDP packet */
#define RTP_RECV_SIZE 65536

/** the IM-Voice session information */
class simv_info_t {

public:
  /** Constructor */
  simv_info_t() { 
    next = NULL;
    oTtsTerminate = false;
    oAsrTerminate = false;
  }

  /** Destructor */
  ~simv_info_t() { }
  
  /*
   * Common Params 
   */

  /** Context for RTP library */
  context rtp_context;

  /** Voice user of this session */
  IptelAddress voice_user;
  
  /** IM user of this session */
  IptelAddress im_user;

  /** Conversion Server info. */
  IptelAddress server;

  /** simvoice <--> IM Session id */
  DString sid;

  /** Call id of the voice session */
  DString cid;

  /** mutex for accessing to RTP socket  */
  pthread_mutex_t rtp_mutex;

  /** condition to signal events: not used */
  pthread_cond_t  rtp_cond;

  /** silence detection energy threshold */
  double vat_th;

  /** next item in the linked list */
  simv_info_t *next;

  /*
   * Params for TTS thread 
   */

  /** Boolean to indicate whether to terminate the thread */
  int  oTtsTerminate;

  /** TTS thread ID */
  pthread_t       tts_thread;

  /** Message to be synthesized */
  /*  Application --> Thread    */
  char *tts_mesg;

  /** Buffer for speech synthesis */
  /* buffer for 20 ms worth linear data */
  int16 tts_buffer[TTS_BUF_SIZE];

  /** Handle for maintaining speech synthesis context */
  ECIHand hECI;  /* Handle to TTS */

  /*
   * Params for ASR thread 
   */

  /** boolean to indicate whether to terminate the thread */
  int  oAsrTerminate;

  /** ASR thread ID */
  pthread_t       asr_thread;

  /** Speech recognized text */
  /*  Thread  --> Application  */
  char *asr_mesg;

};

/** Function Prototypes */
void *TtsRtpSendThread(void *p);
void *AsrRtpRecvThread(void *p);
void sleep_msec(int32 ms);      /* tts.cpp */
int OnEndTalkSpurt(simv_info_t *asr);

