/***************************** FILE HEADER *****************************
#               Copyright (c) 2000 by Columbia University              *
#                        All Rights Reserved                           *
#***********************************************************************
#***********************************************************************
#
#  File Name    : stdtypes.h
#
#  Module       : Audio Conference Server
#
#  Author(s)    : Kundan Singh (kns10@cs.columbia.edu)
#                 Gautam Nair  (gnair@cs.columbia.edu)
#
#  Created      : 28 Apr 2000
#
#  Description  : Common utilities for memory and mutexes.
#                
#  Notes        : 
#
#***********************************************************************
#                Modification History (Reverse Order)
#-----------------------------------------------------------------------
#   Date         Author              Description
#-----------------------------------------------------------------------
#              |             | 
#              |             | 
#-----------------------------------------------------------------------
*/

#ifndef STDTYPES_H
#define STDTYPES_H

/*
 * Macros 
 */
#define AllocateObject(x)  (x *)calloc(1, sizeof(x))
#define AllocateObjects(x,c) (x *)calloc((c), sizeof(x))
#define DeallocateObject(p) if (p) { free(p); p=NULL; }
#define print_rtperror(fxn,err) if ( (err) != RTP_OK) { \
  debug((fxn) ? (fxn) : "RTP", "%s\n", RTPDebugStr()); }

/* Globals */
#ifdef DEBUG

#define LOCK(m)   { debug("pthread_mutex_lock", "entering %s:%d\n",\
                    #m, __LINE__); \
                    pthread_mutex_lock(&(m));   \
                    debug("pthread_mutex_lock", "exiting %s:%d\n",\
                    #m, __LINE__); }
#define UNLOCK(m) { debug("pthread_mutex_unlock", "%s:%d\n",\
                    #m, __LINE__); \
                    pthread_mutex_unlock(&(m)); }
#define COND_WAIT(c,m) { debug("pthread_cond_wait", \
                        "entered %s,%s:%d\n", \
                        #c, #m, __LINE__); \
                    pthread_cond_wait(&(c), &(m)); \
                    debug("pthread_cond_wait", \
                        "exiting %s,%s:%d\n", \
                        #c, #m, __LINE__); }
#define COND_TIMEDWAIT(c,m,n) { debug("pthread_cond_timedwait", \
                        "entered %s,%s:%d\n", \
                        #c, #m, __LINE__); \
                    pthread_cond_timedwait(&(c), &(m), &(n)); \
                    debug("pthread_cond_timedwait", \
                        "exiting %s,%s:%d\n", \
                        #c, #m, __LINE__); }
#define COND_SIGNAL(c) { debug("pthread_cond_signal",\
                        "%s:%d\n", \
                        #c, __LINE__); \
                    pthread_cond_signal(&(c));    }
#else

#define LOCK(m) pthread_mutex_lock(&(m))
#define UNLOCK(m) pthread_mutex_unlock(&(m))
#define COND_WAIT(c,m) pthread_cond_wait(&(c), &(m))
#define COND_TIMEDWAIT(c,m,n) pthread_cond_timedwait(&(c),&(m),&(n))
#define COND_SIGNAL(c) pthread_cond_signal(&(c))

#endif

#endif

/************************** END OF FILE ********************************
*/

